# -*- coding: utf-8 -*-
#
# Copyright (c) 2020~2999 - Cologler <skyoflw@gmail.com>
# ----------
#
# ----------

import os
from typing import Dict
from zipfile import ZipFile
from logging import Logger

def backup_qbittorrent(config_dir: str, destzip_path: str,
                       last_backupzip_path: str=None, logger: Logger=None):

    '''
    - `config_dir` - the qbittorrent config dir, which should contains a `BT_backup` dir.
    - `destzip_path` - the dest zip file path to save the backup files.
    - `last_backupzip_path` - the last backuped zip file to compare if provided.
    '''

    def find_BT_backup():
        BT_backup_dirs = [
            os.path.join(config_dir, 'qBittorrent', 'BT_backup'), # for new version
            os.path.join(config_dir, 'data', 'qBittorrent', 'BT_backup'), # for old version
        ]
        for backup_dir in BT_backup_dirs:
            if os.path.isdir(backup_dir):
                return backup_dir
        logger.error(f'none of {BT_backup_dirs!r} is a dir.')

    BT_backup = os.path.join(config_dir, 'qBittorrent', 'BT_backup')

    BT_backup = find_BT_backup()
    if BT_backup is None:
        return

    files_to_backup: Dict[str, str] = {}

    filenames = set(os.listdir(BT_backup))
    for name in filenames:
        prefix, ext = os.path.splitext(name)
        arc_name = None

        if ext == '.torrent':
            arc_name = 'torrents/' + name
        elif ext == '.fastresume':
            if (prefix + '.torrent') not in filenames:
                # this is a magnet link file
                arc_name = 'magnets/' + name

        if arc_name:
            files_to_backup[arc_name] = os.path.join(BT_backup, name)

    # load exists backups
    if last_backupzip_path:
        with ZipFile(last_backupzip_path, 'r') as last_backup_zip:
            last_backup_torrents = set(last_backup_zip.namelist())
    else:
        last_backup_torrents = set()

    if last_backup_torrents == set(files_to_backup):
        logger.info('nothing changed after last backup.')
        return

    zipsafe = False
    try:
        with ZipFile(destzip_path, 'w') as backup_file:
            for arc_name in sorted(files_to_backup):
                try:
                    backup_file.write(files_to_backup[arc_name], arc_name)
                except:
                    logger.error('error when backuping {}'.format(arc_name), exc_info=True)
                    raise
        zipsafe = True
    finally:
        if not zipsafe and os.path.isfile(destzip_path):
            try:
                os.unlink(destzip_path)
            except:
                pass



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020~2999 - Cologler <skyoflw@gmail.com>
# ----------
# to use this:
#   python -m qblocal_backup qbittorrent/conf dest_location
# ----------

from typing import *
import os
import sys
import traceback
import logging
from datetime import datetime

from . import backup_qbittorrent

def get_timestr():
    dtstr = datetime.now().isoformat()[:19]
    return dtstr.replace(':', '-')

def find_backups(basedir: str, prefix: str):
    '''
    find backups in dir, order by new to old, return the list of fullpath.
    '''

    zips = [f for f in os.listdir(basedir) if os.path.splitext(f)[1] == '.zip' and f.startswith(prefix)]
    if zips:
        zips.sort(reverse=True)
    return [os.path.join(basedir, n) for n in zips]

def main(argv=None):
    if argv is None:
        argv = sys.argv

    logger = logging.getLogger('qblocal.backup')
    logging.basicConfig(level=logging.WARNING, format='%(message)s')
    logger.setLevel(logging.INFO)

    if sys.getdefaultencoding() != 'utf-8':
        logger.warning(f'encoding is {sys.getdefaultencoding()}, which may cause errors.')

    args = argv[1:]
    if len(args) != 2:
        return exit('module qblocal_backup required 2 parameters: <QBITTORRENT_CONF> <DEST_LOCATION>')
    conf_location, dest_dir = args

    try:
        if not os.path.isdir(conf_location):
            return exit(f'{conf_location} is not a dir.')
        if not os.path.isdir(dest_dir):
            os.makedirs(dest_dir, exist_ok=True)

        prefix = 'qbittorrent-'
        destzip_path = os.path.join(dest_dir, f'{prefix}{get_timestr()}.zip')

        exists = find_backups(dest_dir, prefix)
        backup_qbittorrent(
            conf_location, destzip_path,
            last_backupzip_path=next(iter(exists), None),
            logger=logger)

        # cleanup
        exists = find_backups(dest_dir, prefix)
        while len(exists) > 5:
            os.unlink(exists.pop())

    except Exception: # pylint: disable=W0703
        traceback.print_exc()

if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qblocal_backup']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'qblocal-backup',
    'version': '0.1.1',
    'description': 'a local backup tool for qbittorrent without call webui api.',
    'long_description': '# qblocal_backup\n\na local backup tool for qbittorrent without call webui api.\n\n## Usage\n\n``` cmd\npython -m qblocal_backup <QBITTORRENT_CONF> <DEST_LOCATION>\n```\n\n`<DEST_LOCATION>` should be a directory.\n',
    'author': 'Cologler',
    'author_email': 'skyoflw@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Cologler/qblocal_backup-python',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
